package appdata;
use klinstall;

my $APPID = 1194;

my $KLUSER = "kluser";
my $KLGROUP = "klusers";

############################## text strings ##################################
my $SETUPMSG_1 = <<MSG_END;

<B>Setting up protection with Kaspersky Anti-Virus for Proxy server</B>.

The installation program can automatically configure your ICAP-enabled Squid to be protected by Kaspersky Anti-Virus ICAP server.
 
MSG_END

my $SETUPMSG_SQUID_MANUAL_HEAD = <<MSG_END;

<B>Manualy setting up Squid proxy server</B>
MSG_END

my $SETUPMSG_INFO = <<MSG_END;

If you want to configure your Squid later, you can run the configuration script again by running <B>%1</B>.
 
MSG_END

my $SETUPMSG_3 = <<MSG_END;

********************************** WARNING! *********************************
The configuration file %1 you've provided appears to be from a Squid built without ICAP support. Note that you need to get the ICAP-enabled Squid to use it with Kaspersky Anti-Virus ICAP server. Therefore the configuration changes made by the installation may made your Squid setup unusable.

Are you <B>sure</B> that the configuration file you've entered is from the ICAP-enabled Squid?
MSG_END

my $SETUPMSG_4 = <<MSG_END;

<B>Configuring Squid to use Kaspersky Anti-Virus ICAP-Server</B>

Proxy server binary path: %1
Proxy server configuration file: %2

Please enter 'Y' to confirm that you want to protect this proxy server with Kaspersky Anti-Virus. Enter 'N' if proxy server has been detected incorrectly, or if you do not want to protect it.
MSG_END

my $SETUPMSG_5 = <<MSG_END;
Setup was unable to detect an existing installation of a proxy server on your computer. Either it is not installed or has been installed to an unknown location.
If it has been installed, and you know the installation details, enter 'Y'. Otherwise enter 'N' (configuration will be aborted):
MSG_END

my $SETUPMSG_6 = <<MSG_END;

Please enter the full path to your Squid configuration file (or an empty string to abort configuration):
MSG_END

my $SETUPMSG_7 = <<MSG_END;

Please enter the full path to your Squid binary:
MSG_END

my $SETUPMSG_8 = <<MSG_END;

Please choose 1-%1:
MSG_END

my $SETUPMSG_REMOTE_HEAD = <<MSG_END;

<B>Configure address to listen remote connections</B>
 
MSG_END

my $SETUPMSG_REMOTE_INFO = <<MSG_END;
Configuring ICAP-server to listen on %1
MSG_END

my $SETUPMSG_REMOTE_QUERY = <<MSG_END;
Please enter address in <hostname|IP-address>:<port>  format (for example 0.0.0.0:1344 to bind ona all interfeces) or 'cancel' to cancel configuration
MSG_END

my $CFGMARKER = "Added by Kaspersky Anti-Virus installer";
my $CFGMARKER1 = "# $CFGMARKER\n";
my $CFGMARKER2 = "# /$CFGMARKER\n";

# every module should return true to be loaded successfully
1;


# Returns application information.
#
# Should define at least these hash values:
#
sub get_application_info
{
	my %AI;

	# Take install data
	seek (DATA, 0, 0);
	foreach (<DATA>)
	{
		chomp;
		m/^f\s+(.*)$/ and $AI{"FILES"} .= "$1\n"
			or m/^d\s+(.*)$/ and $AI{"DIRS"} .= "$1\n"
				or m/^(\w+)=(.*)$/ and $AI{$1} = $2;
	}

	klinstall::fatal ("Corrupted installation!")
		if !$AI{"INSTROOT"};

	$AI{"ID"} = $APPID;
	$AI{"NAME"} = "Kaspersky Anti-Virus for Proxy Server";
	$AI{"DEFAULT_CONFIG"} = "$AI{CFGPATH}/kav4proxy.conf";

	$AI{"PATH_BASES"} = "$AI{DBROOT}/kav4proxy/bases";
	$AI{"PATH_LICENSES"} = "$AI{DBROOT}/kav4proxy/licenses";

	$AI{"REQUIRE_AV_BASES"} = "0";
	$AI{"REQUIRE_SETUP_KEEPUP2DATE"} = "1";
	$AI{"REQUIRE_LICENSES"} = "1";

	$AI{"TXT_WRTUAB"} = "keepup2date component";
	$AI{"TXT_WRTIL"}  = "$AI{INSTROOT}/bin/kav4proxy-licensemanager -a <keyfile>";

	$AI{"COMP_WEBMIN_MODULE"} = "$AI{SHAREPATH}/contrib/kav4proxy.wbm";
	$AI{"COMP_SETUP_SCRIPT"}  = "$AI{LIBEXEC}/setup/proxy_setup.pl";
	$AI{"COMP_POSTINSTALL_SCRIPT"} = "$AI{LIBEXEC}/setup/postinstall.pl";

	return %AI;
}


# Generic checks
sub app_check_os_ver
{
	my ($arch, $osname, $osver) = @_;

	my %AI = appdata::get_application_info();
	my $req_os_ver  = $AI{'PKG_OS_VER'};
	my $req_os_name = $AI{'PKG_OS_NAME'};

	return "Invalid OS version. This package is only for $req_os_name"
	    if $req_os_ver && ("$osname $osver" !~ /^$req_os_ver/i );

	if ($osname =~ /freebsd/i and $arch =~ /amd64/i) {
	    return "This package requires 32-bit runtime compatibility libraries"
		if ! -f '/libexec/ld-elf32.so.1';
	}

	my $ver55root = ($osname =~ /freebsd/i) ? '/usr/local/share' : '/opt/kav';

	if (-f "$ver55root/5.5/kav4proxy/bin/kavicapserver") {
	    return  "This package is not compatible with older versions of KAV For Proxy Server.\n" .
		    "Please uninstall old version before install this package.";
	}

	return 0;
}

sub app_check_linux_distro { return 0; }
sub app_get_conflict_ids { return; }
sub app_check_required_components { return; }

sub app_check_optional_components
{
	my %AI = appdata::get_application_info();

	# Create kluser/klusers if absent
	my $shell = -f "/sbin/nologin" ? "/sbin/nologin" : "/bin/false";

	klinstall::add_group ($KLGROUP) or return $@;
	klinstall::add_user ($KLUSER, $KLGROUP, $shell, $AI{'DBROOT'}) or return $@;
	return;
}

sub app_start
{
	my %AI = appdata::get_application_info();

	my $cmd = "$AI{RCCMD} start";

	print "Starting ICAP server: ";
	my ($rc, $outmsg) = klinstall::exec_cmd ($cmd);
	print "$outmsg\n";

	#if ($rc != 0) {
	#	print "Executing $cmd failed: $outmsg\n";
	#}
	return $rc == 0;
}

sub app_stop
{
	my %AI = appdata::get_application_info();
	my $cmd = "$AI{RCCMD} stop";

	print "Stopping ICAP server: ";
	my ($rc, $outmsg) = klinstall::exec_cmd ($cmd);
	print "$outmsg\n";

	#my ($rc, $outmsg) = klinstall::exec_cmd ($cmd);
	#if ($rc != 0) {
	#	print "Executing $cmd failed: $outmsg\n";
	#}
}

sub app_is_running
{
	my %AI = appdata::get_application_info();
	my $cmd = "$AI{'RCCMD'} status";

	my ($rc, $outmsg) = klinstall::exec_cmd ($cmd);
	return ($rc >>8) == 0;
}

sub app_checks_preuninstall
{
	return;
}

sub app_is_license_needed
{
	my %AI = get_application_info();
	my $lictool = "$AI{INSTROOT}/bin/kav4proxy-licensemanager";

	# Test whether the valid key file is already installed
	my ($rc, $msg) = klinstall::exec_cmd ("$lictool -q -s");
	return $rc == 0 ? 0 : 1;
}

sub install_licenses
{
	my $licpath = shift;
	my %AI = get_application_info();
	my $lictool = "$AI{INSTROOT}/bin/kav4proxy-licensemanager";

	my $appinfo = $AI{"PATH_LICENSES"} . '/appinfo.dat';
	if ( ! -f $appinfo ) {
		# create empty file
		if (open F, ">$appinfo") {
		close F;
		}
	}
	klinstall::lazy_chown("$KLUSER:$KLGROUP", $appinfo);
	chmod (0640, $appinfo);

	opendir LICDIR, $licpath or return "Could not open directory $licpath: $!";
	my @keyfiles = grep { /\.key$/i && -f "$licpath/$_" } readdir (LICDIR);
	closedir LICDIR;

	return "No license files (files with .key extension) were found in the directory '$licpath'."
		if !@keyfiles;

	# Check all the key files for validity
	my (@validkeys, @errormsgs);

	foreach (@keyfiles)
	{
		# Check whether it is a KL key file and is valid
		my ($rc, $msg) = klinstall::exec_cmd ("$lictool -q -k $licpath/$_");

		$rc != 0 and push @errormsgs, "$_ rejected - not a valid Kaspersky Lab license file."
			or $msg !~ /\bALLOWED=yes\b/ and push @errormsgs, "$_ rejected - not valid for this product."
				or push @validkeys, "$licpath/$_";
	}

	if ( !@validkeys )
	{
		klinstall::output ("\n" . join ("\n", @errormsgs));
		return "None of the Kaspersky Lab license files found is valid for this product.";
	}

	# Try to install the first good key from the list
	@errormsgs = ();
	foreach my $keyfile (@validkeys)
	{
		$keyfile =~ s/\/+/\//g; # remove extra /
		my ($rc, $msg) = klinstall::exec_cmd ("$lictool -q -a $keyfile");
		if ( $rc == 0 )
		{
			klinstall::output ("The license file %1 has been installed.", $keyfile);
			return;
		}

		push @errormsgs, "Error installing license file $keyfile: $msg";
	}

	klinstall::output ("\n" . join ("\n", @errormsgs)) if @errormsgs;
	return "None of the Kaspersky Lab license files found can be installed for this product.";
}

sub app_modify_config_file
{
	my %AI = appdata::get_application_info();
	klinstall::copy_file("$AI{DEFAULT_CONFIG}.default", $AI{'DEFAULT_CONFIG'})
	    if $AI{'PKG_OS_VER'} =~ /^FreeBSD/i && ! -f $AI{'DEFAULT_CONFIG'};
}

sub setup_http_proxy
{
	my $addr = shift;
	my %AI = get_application_info();
	klinstall::warning ("Could not set up proxy address: ", klinstall::get_ini_error())
		if !klinstall::update_ini_file ( $AI{"DEFAULT_CONFIG"}, "updater.options", ("ProxyAddress" => $addr, "UseProxy" => "yes") );
}

sub update_AV_database
{
	my %AI = get_application_info();
	my $updater_cmd = "$AI{INSTROOT}/bin/kav4proxy-keepup2date -k -c $AI{DEFAULT_CONFIG}";

	# Escape shell args
	$updater_cmd =~ s/([\&;\`'\\\|"*?~<>^\(\)\[\]\{\}\$\n\r])/\\$1/g; #'

	my $retcode = system ("$updater_cmd");

	system("sh $AI{LIBEXEC}/setup/keepup2date.sh -prompt-install");
	
	# return code 1 is 256. perldoc -f wait
	return (defined $retcode && ($retcode eq 0 || $retcode eq 256)) ? 1 : 0;
}

sub set_value
{
	my ($ref, $var, $val) = @_;

	return if (${$ref} =~ s/^(#$var\s+.+)/$1/m);
	return if (${$ref} =~ s/^$var\s+.+/$var $val/m);
}

sub add_or_replace
{
	my ($ref, $var, $val) = @_;

	return if (${$ref} =~ s/^$var\s+.+/$var $val/m);
	return if (${$ref} =~ s/^(#\s*$var\s+.+\n(#.*\n)*)/$1$var $val\n/m);

	return if (${$ref} =~ s/($CFGMARKER1.*\n)($CFGMARKER2)/$1$var $val\n$2/sm);
	${$ref} .= "\n${CFGMARKER1}${var} $val\n${CFGMARKER2}";
}

sub add_or_replace_named
{
	my ($ref, $var, $name, $val) = @_;

	return if (${$ref} =~ s/^$var\s+$name\s+.+/$var $name $val/m);
	return if (${$ref} =~ s/^(#\s*$var\s+.+\n(#.*\n)*)/$1$var $name $val\n/m);

	return if (${$ref} =~ s/^($CFGMARKER1.*\n)($CFGMARKER2)/$1$var $name $val\n$2/sm);
	${$ref} .= "\n${CFGMARKER1}$var $name $val\n${CFGMARKER2}";
}

sub remove_named
{
	my ($ref, $var, $name) = @_;

	${$ref} =~ s/^(#?\s*$var\s+$name\s+.+\n)//m;
}

sub find_installed_proxy
{
	my @found;

	my @squid_paths = (
		{
			'conf' => '/usr/local/squid/etc/squid.conf',
			'bin'  => '/usr/local/squid/sbin/squid',
		},
		{
			'conf' => '/usr/local/etc/squid/squid.conf',
			'bin'  => '/usr/local/sbin/squid',
		},
		{
			'conf' => '/etc/squid/squid.conf',
			'bin'  => '/usr/sbin/squid',
		},
	);

	# Find the squid proxy server the customer is using
 	foreach my $paths (@squid_paths) {
		next if (! -x $paths->{'bin'}) or (! -f $paths->{'conf'});
		$paths->{'type'} = 'squid';
		$paths->{'title'} = 'Squid ('.$paths->{'conf'}.')';
		push @found, $paths;
	}

	return @found;
}

sub configure_squid
{
	my %APPINFO = appdata::get_application_info();
	
	my $data = shift;

	return if klinstall::ask_boolean ('KAV4PROXY_CONFIRM_FOUND', 'Y', $SETUPMSG_4, ${$data}{'bin'}, ${$data}{'conf'}) != 1;

	$squidbinfile = $data->{'bin'};
	$squidcfgfile = $data->{'conf'};
	
	if ( !open F, "<$squidcfgfile" ) {
		klinstall::output ("Could not read the configuration file %1: %2\n", $squidcfgfile, $!);
		return;
	}
	$squidcfgcontent = join ('', <F>);
	close F;

	# Make a backup
	open F, ">$squidcfgfile.kavbackup" or klinstall::fatal ("Could not create backup file $squidcfgfile.kavbackup: $!");
	print F $squidcfgcontent;
	close F;

	my $squid_ver = `$squidbinfile -v`;
	$squid_ver = ($squid_ver =~ /Version\s+(\S+)/) ? $1 : 'unknown';

	# Enable or add the ICAP options
	add_or_replace (\$squidcfgcontent, "icap_enable", "on");

	add_or_replace (\$squidcfgcontent, "icap_send_client_ip", "on");

	add_or_replace_named (\$squidcfgcontent, "icap_service", "is_kav_req",  "reqmod_precache 0 icap://localhost:1344/av/reqmod");
	add_or_replace_named (\$squidcfgcontent, "icap_service", "is_kav_resp", "respmod_precache 0 icap://localhost:1344/av/respmod");

	if ($squid_ver =~ /^2\./) {
		# workarround for bug in Squid 2.x - reqmod is not sent when it's class defined after respmod
		# but works stable if both defined in one class
		add_or_replace_named (\$squidcfgcontent, "icap_class",   "ic_kav", "is_kav_req is_kav_resp");
		add_or_replace_named (\$squidcfgcontent, "icap_access",  "ic_kav", "allow all");
	} else {
		add_or_replace_named (\$squidcfgcontent, "acl",		  "acl_kav_GET", "method GET");
		add_or_replace_named (\$squidcfgcontent, "icap_class",   "ic_kav_req",  "is_kav_req");
		add_or_replace_named (\$squidcfgcontent, "icap_class",   "ic_kav_resp", "is_kav_resp");

		add_or_replace_named (\$squidcfgcontent, "icap_access",  "ic_kav_resp", "allow all");
		add_or_replace_named (\$squidcfgcontent, "icap_access",  "ic_kav_req",  "allow all !acl_kav_GET");
	}

	open F, ">$squidcfgfile" or klinstall::fatal ("Could not write the Squid configuration file $squidcfgfile: $!");
	print F $squidcfgcontent;
	close F;

	my %appdata = klinstall::read_app_info ($APPID);
	$appdata{'SQUID_CONF'} = $squidcfgfile;
	$appdata{'SQUID_BIN'}  = $squidbinfile;
	klinstall::write_app_info ($APPID, %appdata);
	
	return 1;
}

sub configure_proxy_none_cb
{
	my $data = shift;
	return;
}

sub configure_proxy_remote_cb
{
	my $data = shift;

	my $listen;

	my %APPINFO = appdata::get_application_info();

	require Socket;

	klinstall::output( $SETUPMSG_REMOTE_HEAD);

	while (1) {
		$listen = klinstall::ask_question('KAV4PROXY_SETUP_LISTENADDRESS', '0.0.0.0:1344', $SETUPMSG_REMOTE_QUERY);

		return if ($listen =~ /^\s*CANCEL\s*$/i);

		if ($listen =~ /^(.*):(\d+)$/) {
			my ($host, $port) = ($1, $2);

			my $ip = Socket::inet_aton( $host );

			if (!length($ip)) {
				klinstall::output("\n'%1' is not valid hostname or IP-address!\n", $host);
				next;
			}

			if ($port <= 0 or $port > 65535) {
				klinstall::output("\n'%1' is not valid port number!\n", $host);
				next;
			}

			last;
		} else {
			klinstall::output("\n'%1' is not in valid format. Please specify in hostname:port format\n", $listen);
		}
	}

	klinstall::output($SETUPMSG_REMOTE_INFO,  $listen);

	# Configure ListenAddress in section [icapserver.network]
	my $ok = klinstall::update_ini_file ( $APPINFO{"DEFAULT_CONFIG"}, "icapserver.network", ("ListenAddress" => $listen) );
	if ($ok) {
		return 1;
	} else {
		klinstall::warning ("Could not set up listen address: %1", klinstall::get_ini_error())
	}

	return;
}

sub configure_proxy_squid_manual_cb
{
	my $data = shift;

	klinstall::output($SETUPMSG_SQUID_MANUAL_HEAD);

	my ($squidcfgfile, $squidbinfile);

	while ( 1 ) {
		$squidcfgfile = klinstall::ask_question ('KAV4PROXY_SETUP_CONFPATH', "", $SETUPMSG_6);
		$squidcfgfile =~ s/\s//g;

		return if ( !$squidcfgfile );

		# check for given path exists
		if ( ! -f $squidcfgfile ) {
			klinstall::output ("File not found: %1", $squidcfgfile);
			next;
		}
	
		$squidbinfile = klinstall::ask_question ('KAV4PROXY_SETUP_BINPATH', '', $SETUPMSG_7);
		$squidbinfile =~ s/\s//g;
		if ( !$squidbinfile ) {
			next;
		}

		# check for given path exists
		if ( ! -f $squidbinfile || ! -x $squidbinfile ) {
			klinstall::output ("File not found, or not an executable: %1", $squidbinfile);
			next;
		}

		if ( !open F, "<$squidcfgfile" ) {
			klinstall::output ("Could not read the configuration file %1: %2\n", $squidcfgfile, $!);
			next;
		}
		$squidcfgcontent = join ('', <F>);
		close F;

		$squidcfgcontent =~ s/\r//g;

		next if $squidcfgcontent !~ /^#?\s*icap_enable/m
			&& klinstall::ask_boolean ("KAVICAP_SETUP_NONICAPCFG", "N", $SETUPMSG_3, $squidcfgfile) == 0;

		last;
	}

	return configure_squid({
		'bin'  => $squidbinfile,
		'conf' => $squidcfgfile,
	});
}

sub configure_proxy_squid_cb
{
	my $data = shift;
	return configure_squid($data);
}

sub configure_proxy
{
	my %APPINFO = appdata::get_application_info();

	my $result = undef;
	my @found;

	push @found, {
		'type'  => 'none',
		'title' => 'No integration',
	};
	push @found, {
		'type'  => 'remote',
		'title' => 'Configure to work with remote proxy',
	};
	push @found, {
		'type'  => 'squid_manual',
		'title' => 'Configure Squid manually',
	};
	push @found, find_installed_proxy();

	klinstall::output ($SETUPMSG_1);

	while (1) {
		for ($i=0; $i < scalar(@found); $i++) {
			my $data = $found[$i];
			klinstall::output("  <B>%1)</B> %2", $i+1, $data->{'title'});
		}

		my $sel = klinstall::ask_question('KAV4PROXY_SETUP_TYPE', "", $SETUPMSG_8, scalar(@found));

		if ($sel =~ /^\d+$/ and $sel > 0 and $sel <= scalar(@found)) {
			my $data = $found[$sel-1];
			my $subname = 'configure_proxy_' . ${data}->{'type'} . '_cb';
			if (&{$subname}($data)) {
				$result = $data;
			}
			last;
		} else {
			
		}
	}

	klinstall::output ($SETUPMSG_INFO, $APPINFO{'COMP_SETUP_SCRIPT'})
		if !$result;

	return $result;
}

sub app_register
{
	klinstall::check_for_root ("setup script");

	my $upgrade = shift;
	my $configured;

	if (!$upgrade) {
	    $configured = configure_proxy();
	}

	app_stop() if app_is_running();
	app_start();

	if ($configured && defined $configured->{'bin'}) {
		# reload Squid
		print "Reconfigure Squid - ";
		system ($configured->{'bin'} . ' -k reconfigure');
		print "success\n" if ( ($? >> 8) == 0);
	}
}

sub unconfigure_squid
{
	my %appdata = klinstall::read_app_info ($APPID);
	my $squidcfgfile = $appdata{'SQUID_CONF'};
	my $squidbinfile = $appdata{'SQUID_BIN'};

	return if !defined($squidcfgfile) or ($squidcfgfile eq '');

	if ( open F, "<$squidcfgfile" ) {
		my $squidcfgcontent = join ("", <F>);
		close F;

		# Make a backup
		open F, ">$squidcfgfile.kavremove" or klinstall::fatal ("Could not create backup file $squidcfgfile.kavbackup: $!");
		print F $squidcfgcontent;
 		close F;

		set_value (\$squidcfgcontent, "icap_enable", "off");

		#remove all texts marked as added by installer
		$squidcfgcontent =~ s/^(\n$CFGMARKER1.*$CFGMARKER2)//sm;

		remove_named (\$squidcfgcontent, "acl",		  "acl_kav_GET");
		remove_named (\$squidcfgcontent, "icap_service", "is_kav_req");
		remove_named (\$squidcfgcontent, "icap_service", "is_kav_resp");
		remove_named (\$squidcfgcontent, "icap_class",   "ic_kav_req");
		remove_named (\$squidcfgcontent, "icap_class",   "ic_kav_resp");
		remove_named (\$squidcfgcontent, "icap_access",  "ic_kav_req");
		remove_named (\$squidcfgcontent, "icap_access",  "ic_kav_resp");

		# remove squid 2.x definitions
		remove_named (\$squidcfgcontent, "icap_class",   "ic_kav");
		remove_named (\$squidcfgcontent, "icap_access",  "ic_kav");

		if (open F, ">$squidcfgfile") {
			print F $squidcfgcontent;
			close F;
		}

		# reload Squid
		print "Reconfiguring Squid - ";
		system ($squidbinfile . ' -k reconfigure');
		print "success\n" if ( ($? >> 8) == 0);
	} else {
		klinstall::output ("Could not read the configuration file %1: %2\n", $squidcfgfile, $!);
	}
}

sub app_unregister
{
	my $upgrade = shift;

	my %AI = appdata::get_application_info();

	if (not $upgrade) {
	    system ("sh $AI{LIBEXEC}/setup/keepup2date.sh -uninstall");
    	    unconfigure_squid();
	}
}

__DATA__
VER=5.5.41
INSTROOT=/opt/kaspersky/kav4proxy
RCCMD=/etc/init.d/kav4proxy
CFGPATH=/etc/opt/kaspersky
DBROOT=/var/opt/kaspersky
LIBSPATH=/opt/kaspersky/kav4proxy/lib
LIBEXEC=/opt/kaspersky/kav4proxy/lib/bin
SHAREPATH=/opt/kaspersky/kav4proxy/share
PKG_OS_VER=
PKG_OS_NAME=
