#!/usr/bin/perl -w
#
# Installation script for Kaspersky Lab products for Unix (R)
# Copyright (C) Kaspersky Lab 2003

# Really useful when script is run from the relative path
BEGIN { unshift @INC, $1 if $0 =~ /(.*)[\/]/; }

use strict;
use klinstall;
use appdata;

############################## text strings ##################################
my $INSTALL_MSG_1 = <<INSTALL_MSG_1_END;
%1 cannot be installed on this system: %2.
INSTALL_MSG_1_END

my $INSTALL_MSG_2 = <<INSTALL_MSG_2_END;
%1 was not tested under this Linux distribution.
INSTALL_MSG_2_END

my $INSTALL_MSG_3 = <<INSTALL_MSG_3_END;
%1 was not tested under this version of %2.
INSTALL_MSG_3_END

my $INSTALL_MSG_4 = <<INSTALL_MSG_4_END;
%1 version %2 is already installed.
INSTALL_MSG_4_END

my $INSTALL_MSG_5 = <<INSTALL_MSG_5_END;
%1 is not compatible with already installed %2 because of %3.

You should uninstall %2 before installing %1.
INSTALL_MSG_5_END

my $INSTALL_MSG_6 = <<INSTALL_MSG_6_END;
%1 requires %2 version %3 or greater to work.
INSTALL_MSG_6_END

my $INSTALL_MSG_7 = <<INSTALL_MSG_7_END;
%1 needs %2 to perform %3.
INSTALL_MSG_7_END

my $INSTALL_MSG_9 = <<INSTALL_MSG_9_END;
License file (a file with .key extension) is your personal license key. You need to install it to use the application. 
To install it right now, just enter the path to the location of your license file (enter an empty string to continue without key file installation):
INSTALL_MSG_9_END

my $INSTALL_MSG_10 = <<INSTALL_MSG_10_END;

Please enter another path to the location of your license file (enter an empty string to continue without key file installation):
INSTALL_MSG_10_END

my $INSTALL_MSG_12 = <<INSTALL_MSG_12_END;
You must install a valid Kaspersky Lab license file to use the application.
To do this, run <B>%1</B>.
INSTALL_MSG_12_END

my $INSTALL_MSG_13 = <<INSTALL_MSG_13_END;
Latest anti-virus bases are an essential part of your anti-virus protection. Do you want to download the latest anti-virus bases right now to insure your application is up to date? (If you answer 'yes', make sure you are connected to the Internet):
INSTALL_MSG_13_END

my $INSTALL_MSG_14 = <<INSTALL_MSG_14_END;
Latest anti-virus bases are an essential part of your anti-virus protection. You must run <B>%1</B> to download the latest anti-virus bases before using the application.
INSTALL_MSG_14_END

my $INSTALL_MSG_15 = <<INSTALL_MSG_15_END;
Latest anti-virus bases are an essential part of your anti-virus protection, and without them the application will not function. You must run %1 to download the latest anti-virus bases before using the application, and then run <B>%2</B> configuration script to set up the %3.
INSTALL_MSG_15_END

my $INSTALL_MSG_16 = <<INSTALL_MSG_16_END;

%1 is installed.
Configuration file was installed in %2
Binaries were installed in %3

INSTALL_MSG_16_END

my $INSTALL_MSG_17 = <<INSTALL_MSG_17_END;
If you use an http proxy server to access the Internet, you need to tell the %1 KeepUp2Date component about it. Please enter the address of your http proxy server in one of the following forms, http://proxyIP:port or http://user:pass\@proxyIP:port. If you don\'t have or need a proxy server to access the Internet, enter 'no' here
INSTALL_MSG_17_END

my $INSTALL_MSG_18 = <<INSTALL_MSG_18_END;
%1 has been installed successfully but needs to be properly configured before using. 
Unfortunately, RPM is not able to run scripts interactively, so please run <B>%2</B> script by yourself to configure it.
INSTALL_MSG_18_END

my $INSTALL_MSG_19 = <<INSTALL_MSG_19_END;
Cannot update product\'s file %1. Please resolve the problem and reinstall the package.
INSTALL_MSG_19_END

my $WEBMIN_MSG_1 = <<WEBMIN_MSG_1_END;

Default Webmin configuration file was not found. This means that either Webmin is not installed at all, or is installed into a non-default location.

Webmin (www.webmin.com) is a web-based interface for system administration for various Unix components. If you install it, you'll be able to configure and use Kaspersky Anti-Virus through the web interface. If you want to use this functionality, but haven't installed Webmin yet, you can skip this stage and install this module later using Webmin\'s built-in installation procedure.
If you have Webmin installed in a non-default path, please enter the path to the location of the Webmin configuration file, or leave blank to skip?
WEBMIN_MSG_1_END

my $WEBMIN_MSG_2 = <<WEBMIN_MSG_2_END;
If you want to use this module later, you can install it using Webmin\'s own installation procedure. The module will be placed in %1.
WEBMIN_MSG_2_END

my $WEBMIN_MSG_3 = <<WEBMIN_MSG_3_END;
Webmin configuration file %1 is incorrect. 
Please enter another path to the Webmin configuration file, or leave blank to skip?
WEBMIN_MSG_3_END

my $WEBMIN_MSG_4 = <<WEBMIN_MSG_4_END;
Webmin module installed successfully. Check it on 'Others' web page.
Depending on Webmin configuration, you may also need to enable the Kaspersky Anti-Virus Webmin module. To do this, on the Webmin main page go to Webmin -> Webmin users -> <username> -> set the check box on <B>%1</B>.
WEBMIN_MSG_4_END

my $WEBMIN_MSG_5 = <<WEBMIN_MSG_5_END;
failed to install Webmin module: %1
WEBMIN_MSG_5_END


############################### handlers ####################################
sub sigint_handler
{
	print STDERR "Interrupted.\n";
	exit 0;
}


############################### routines ####################################
sub setup_webmin_module
{
	my (%APPINFO) = @_;

	my $webminroot;
	my $webmincfg = -f "/usr/local/etc/webmin/miniserv.conf" ? "/usr/local/etc/webmin/miniserv.conf" : "/etc/webmin/miniserv.conf";
	my $webmindircfg = klinstall::internal_dirname ($webmincfg);

	while ( !-f $webmincfg )
	{
		$webmincfg = klinstall::ask_question ("CONFIGURE_WEBMIN_ASKCFGPATH", "", $WEBMIN_MSG_1);

		if ( !$webmincfg )
		{
			klinstall::output ($WEBMIN_MSG_2, $APPINFO{"COMP_WEBMIN_MODULE"});
			return;
		}
	}

	while ( 1 )
	{
		if ( open FI, "<$webmincfg" )
		{
			my @roots = grep (/^root=.*/, <FI>);
			close FI;

			$webminroot = $1 if @roots && $roots[0] && $roots[0] =~ /^root=(.*)\s*$/;
		}

		last if $webminroot;
		$webmincfg = klinstall::ask_question ("CONFIGURE_WEBMIN_ASKANOTHERCFGPATH", "", $WEBMIN_MSG_3, $webmincfg);
	}

	my ($rc, $outmsg) = klinstall::exec_cmd ("perl $webminroot/install-module.pl $APPINFO{COMP_WEBMIN_MODULE} $webmindircfg");

	if ( $rc == 0 )
	{
		klinstall::output ($WEBMIN_MSG_4, $APPINFO{"NAME"});
		return 1;
	}
	
	klinstall::warning ($WEBMIN_MSG_5, $outmsg);
	return;
}



############################### main() ######################################

	my %cmdline_selected;
	my %cmdline_options_list = 
			("debug" => "Enable debug mode",
			 "force" => "Force installation even if there's a conflict",
			 "upgrade" => "Perform upgrade",
			 "help"  => "This help message");

	# handle Ctrl-C
	$SIG{'INT'} = "sigint_handler";

	# check for root
	klinstall::check_for_root ("installation");

	# Analyze extra command-line options. 1st arg could be 'used logic'
	my ($logic, $pkgmgr) = klinstall::get_work_logic($0, \@ARGV);

	# parse command-line options and display help only if run manually
		my @argslist = @ARGV;
		push @argslist, split ('\s+', $ENV{"KLINSTALL"}) if $ENV{"KLINSTALL"};

		foreach (@argslist)
		{
			next if !$_;

			my $arg = $_;
			$arg = $1 if $arg =~ /^--(.*)$/;

			if ( !$cmdline_options_list{$arg} )
			{
				print "Unsupported command line option: $_\n\n";
				display_cmdline_options(%cmdline_options_list);
				exit 1;
			}

			$cmdline_selected{$arg} = 1;
		}

	if ( !$logic )
	{
		# --help
		if ( $cmdline_selected{"help"} )
		{
			display_cmdline_options();
			exit 1;
		}
	}

	# pre-install logic
	if ( !$logic || $logic eq "pre" )
	{
		# load appdata module
		my %APPINFO = appdata::get_application_info();

		# check for allowed arch/OS/version
		my ($arch, $name, $rev) = klinstall::get_os_info();

		klinstall::debug "Got info: $arch, os: $name, ver: $rev" 
			if $cmdline_selected{"debug"};

		# Check for arch, OS etc.
		my $os_ver_reason = appdata::app_check_os_ver ($arch, $name, $rev);
		klinstall::fatal ($INSTALL_MSG_1, $APPINFO{"NAME"}, $os_ver_reason)
			if $os_ver_reason;

		# check for Linux distribution name
		my %linux_distro = klinstall::get_linux_info();
		if ( $linux_distro{"NAME"} )
		{
			my $res = appdata::app_check_linux_distro (lc $linux_distro{"NAME"}, $linux_distro{"VER"});

			$res == 1 and klinstall::warning ($INSTALL_MSG_2, $APPINFO{"NAME"})
				or $res == 2 and klinstall::warning ($INSTALL_MSG_3, $APPINFO{"NAME"}, $linux_distro{"NAME"})
		}

		# Check for required components
		my @req_comps = appdata::app_check_required_components();
		klinstall::fatal ($INSTALL_MSG_6, $APPINFO{"NAME"}, $req_comps[0]{"NAME"}, $req_comps[0]{"VER"})
			if @req_comps;

		# Check for optional components
		my @opt_comps = appdata::app_check_optional_components();
		foreach (@opt_comps)
		{
			my %rec = %{$_};
			klinstall::warning ($INSTALL_MSG_7, $APPINFO{"NAME"}, $req_comps[0]{"NAME"}, $req_comps[0]{"WHAT2DO"});
		}

		# Check whether the application is already installed - only for tar.gz
		if ( !$pkgmgr && !$cmdline_selected{"force"} )
		{
			my %already_installed = klinstall::read_app_info ($APPINFO{"ID"});
			klinstall::fatal ($INSTALL_MSG_4, $APPINFO{"NAME"}, $already_installed{"VERSION"}) if %already_installed;

            # Check for conflicting applications
			my %applist = appdata::app_get_conflict_ids ();
			foreach (keys %applist)
			{
				my %ai = klinstall::read_app_info ($_);
				next if !%ai;

				klinstall::fatal ($INSTALL_MSG_5, $APPINFO{"NAME"}, $ai{"NAME"}, $applist{"$_"});
			}
		}
	}

	my (@installed_files, @installed_dirs, %uniqmatch);

	# install (copy-files) logic
	if ( !$logic )
	{
		# load appdata module
		my %APPINFO = appdata::get_application_info();

		foreach (split ("\n", $APPINFO{"DIRS"}))
		{
			next if !/^(\d+)\s+([\w\:]+)\s+(.*)$/;
			my ($mode, $own, $dest) = ($1, $2, $3);
			my $realdest = $dest;

			if ( $dest =~ /^(\w+)\/(.*)$/ )
			{
				if ( !defined $APPINFO{$1} )
				{
					klinstall::warning ("Bug in installer script: file $2 should be installed in unknown directory $1.");
					next;
				}

				$realdest = defined $2 && length $2 ? "$APPINFO{$1}/$2" : "$APPINFO{$1}";
			}

			next if -d $realdest;
			push @installed_dirs, $dest;

			klinstall::output ("Creating directory $realdest\n");
			klinstall::install_path ($realdest, $mode, $own, \@installed_dirs);
		}

		foreach (split ("\n", $APPINFO{"FILES"}))
		{
			next if !/^(\d+)\s+([\w\:]+)\s+(.*?)\s+(.*)$/;
			my ($mode, $own, $dest, $src) = ($1, $2, $3, $4);
			my $realdest = $dest;

			if ( $dest =~ /^(\w+)\/(.*)$/ )
			{
				if ( !defined $APPINFO{$1} )
				{
					klinstall::warning ("Bug in installer script: file $2 should be installed in unknown directory $1.");
					next;
				}

				$realdest = "$APPINFO{$1}/$2";
			}

			if ( $realdest =~ /(.*)\/(.*?)$/ && !-d $1 )
			{
				my $msg = klinstall::make_path ($1, 0755, \@installed_dirs);
				klinstall::fatal ($msg) if $msg;
			}

			push @installed_files, $dest;
			klinstall::output ("Installing file $realdest\n");
			klinstall::install_file ($src, $realdest, $mode, $own);
		}

		%uniqmatch = ();
		@installed_files = grep { ! $uniqmatch{$_} ++ } @installed_files;
		%uniqmatch = ();
		@installed_dirs = grep { ! $uniqmatch{$_} ++ } @installed_dirs;

		print ("\n");
	}

	# post-install logic
	if ( !$logic || $logic eq "post" )
	{
		my %APPINFO = appdata::get_application_info();
		my %INSTINFO = klinstall::read_app_info ($APPINFO{"ID"});

		# Store generic appinfo
		my %stored_info;
		$stored_info{"ID"} = $APPINFO{"ID"};
		$stored_info{"NAME"} = $APPINFO{"NAME"};
		$stored_info{"VERSION"} = $APPINFO{"VER"};
		$stored_info{"DEFAULTCONFIG"} = $APPINFO{"DEFAULT_CONFIG"};
		$stored_info{"USED_PKGMGR"} = $pkgmgr if $pkgmgr;
		$stored_info{"INSTROOT"} = $APPINFO{"INSTROOT"};
		$stored_info{"PATH_BASES"} = $APPINFO{"PATH_BASES"} if $APPINFO{"PATH_BASES"};
		$stored_info{"PATH_LICENSES"} = $APPINFO{"PATH_LICENSES"} if $APPINFO{"PATH_LICENSES"};
		if ($pkgmgr eq 'rpm') {
		    # store upgrade flag for upgrade from older installer what does not use
		    # --upgrade cmd line option due uninstall
		    $stored_info{"IS_UPDATE"} = $cmdline_selected{"upgrade"} ? 1 : 0;
		}

		if ( @installed_files || @installed_dirs )
		{
			$stored_info{"FILE"} = \@installed_files if @installed_files;
			$stored_info{"DIR"} = \@installed_dirs if @installed_dirs;
		}

		klinstall::fatal ($INSTALL_MSG_19, klinstall::app_info_filename())
		    if !klinstall::write_app_info ($APPINFO{"ID"}, %stored_info);

		if ( $pkgmgr eq "rpm" )
		{
			klinstall::output ($INSTALL_MSG_18, $APPINFO{"NAME"}, $APPINFO{"COMP_POSTINSTALL_SCRIPT"});
			exit 0;
		}

		appdata::app_modify_config_file();

		my $av_bases_updated = 0;
		my $postinstall_message = "";

		# Install licenses (key files)
		if ( $APPINFO{"REQUIRE_LICENSES"}  && appdata::app_is_license_needed() )
		{
			klinstall::output ("Installing license files.");

			my $licpath = klinstall::ask_question ("CONFIGURE_ENTER_KEY_PATH", "", $INSTALL_MSG_9);
			if ( $licpath )
			{
				while ( 1 )
				{
					my $errmsg = appdata::install_licenses ($licpath);
					last if !$errmsg;

					klinstall::output ("%1", $errmsg);

					$licpath = klinstall::ask_question ("CONFIGURE_ENTER_KEY_ANOTHER_PATH", "", $INSTALL_MSG_10);
					last if !$licpath;
				}
			}

			$postinstall_message .= "\n" . klinstall::translate_string ($INSTALL_MSG_12, $APPINFO{"TXT_WRTIL"})
				if !$licpath;
		}

		my $proxy_is_configured = 0;
		if ( defined(&appdata::app_is_proxy_configured) ) {
		    $proxy_is_configured = appdata::app_is_proxy_configured();
		}

		# Configure the KeepUp2Date component
		if ( $APPINFO{"REQUIRE_SETUP_KEEPUP2DATE"} && !$proxy_is_configured)
		{
			klinstall::output ("\nConfiguring KeepUp2Date proxy settings.");
			while ( 1 )
			{
				my $http_proxy = klinstall::ask_question ("CONFIGURE_KEEPUP2DATE_ASKPROXY",
						"no",
						$INSTALL_MSG_17, 
						$APPINFO{"NAME"});

				last if !$http_proxy || $http_proxy eq "no";
				if ( $http_proxy =~ /^(http:\/\/.*?:.*?\@[\w\-\d\.:]+)\/?$/
				|| $http_proxy =~ /^(http:\/\/[\w\-\d\.:]+)\/?$/ )
				{
					appdata::setup_http_proxy($1);
					last;
				}
			}
		}

		# Set up the AV bases
		if ( $APPINFO{"PATH_BASES"} )
		{
			my $updates_path = $ENV{"KLUPDATES"} ? $ENV{"KLUPDATES"} : ( -d  "updates" ? "updates" : "../updates" );
			if ( !klinstall::install_AV_bases ($updates_path, $APPINFO{"PATH_BASES"}) )
			{
				if ( klinstall::ask_boolean ("CONFIGURE_RUN_KEEPUP2DATE", "yes", $INSTALL_MSG_13) )
				{
					# run keepup2date
					$av_bases_updated = 1 if appdata::update_AV_database();
				}

				$postinstall_message .= "\n" . klinstall::translate_string ($APPINFO{"REQUIRE_AV_BASES"} ? $INSTALL_MSG_15 : $INSTALL_MSG_14, 
							$APPINFO{"TXT_WRTUAB"},
							$APPINFO{"COMP_SETUP_SCRIPT"},
							$APPINFO{"NAME"}) 
					if !$av_bases_updated;
			}
			else
			{
				$av_bases_updated = 1;
			}
		}

		setup_webmin_module (%APPINFO) if $APPINFO{"COMP_WEBMIN_MODULE"};

		# run setup.pl
		if ( !$APPINFO{"REQUIRE_AV_BASES"} || $av_bases_updated )
		{
			system ("perl $APPINFO{COMP_SETUP_SCRIPT}");
			exit $? if ($? & 127) != 0;
		}

		klinstall::output ($INSTALL_MSG_16, $APPINFO{"NAME"}, $APPINFO{"DEFAULT_CONFIG"}, "$APPINFO{INSTROOT}/bin");
		klinstall::output ("\n$postinstall_message") if $postinstall_message;
	}

	exit 0;
